"""
Preprocessor used as additive scrambler in kaitai struct
"""


def pop_count(inp):
    """
    Function to determine the population count in the bit-stream
    "borrowed" in gnuradio
    """
    pop_cnt = inp - ((inp >> 1) & 0o33333333333) - ((inp >> 2) & 0o11111111111)
    return ((pop_cnt + (pop_cnt >> 3)) & 0o30707070707) % 63


def additive_scramble(in_bytes, poly, seed, length):
    """
    Function for implementing an additive scrambler (aka synchronous or
    linear-feedback shift register scrambler) and descrambler.
    """
    in_bits = byte2bit(in_bytes)
    bit_len = len(in_bits)
    out_bits = [0] * bit_len
    lfsr = LFSR(poly, seed, length)
    reg_bit = 0
    for j in range(bit_len):
        reg_bit = lfsr.next_bit()
        out_bits[j] = in_bits[j] ^ reg_bit
    return bit2byte(out_bits)


def byte2bit(in_bytes):
    """
    Function for converting a byte stream into the corresponding bit stream.
    Original Author: Filippo Valmori
    """
    in_len = len(in_bytes)
    out_len = (in_len << 3)
    out_bits = [0] * out_len
    for j in range(out_len):
        byte_idx = (j >> 3)
        bit_idx = 7 - (j % 8)
        if (in_bytes[byte_idx] >> bit_idx) % 2:
            out_bits[j] = 1
    return out_bits


def bit2byte(in_bits):
    """
    Function for converting a bit stream into the corresponding byte stream.
    Original Author: Filippo Valmori
    """
    in_len = len(in_bits)
    out_len = (in_len >> 3)
    out_bytes = [0] * out_len
    for j in range(in_len):
        if in_bits[j]:
            byte_idx = (j >> 3)
            bit_idx = 7 - (j % 8)
            out_bytes[byte_idx] += (1 << bit_idx)
    return out_bytes


class LFSR:
    """
    Linear feedback shift register
    """
    def __init__(self, mask, seed, length):
        """
        Initialisation of the LFSR
        Obvious, isn't it!? ;-)

        :param mask: the polynomial mask
        :param seed: the initial state
        :param lenght: the legth of the register (length = 7 == 8 bits!)
        """
        self.mask = mask
        self.reg = seed
        self.length = length

    def next_bit(self):
        """
        Returns the next bit of the LFSR shift
        """
        output = self.reg & 1
        newbit = pop_count(self.reg & self.mask) % 2
        self.reg = ((self.reg >> 1) | (newbit << self.length))
        return output

    def next_byte(self):
        """
        Returns the next whole byte
        """
        output = self.reg & 0xFF
        for _ in range(8):
            self.next_bit()
        return output


class Scrambler(object):  # pylint: disable=too-few-public-methods
    """
    Scrambler preprocessor class
    """
    def __init__(self, poly, seed, length):
        """
        Inititalize the scrambler

        :param poly: the polynom
        :param seed: the initial state of the LFSR
        """

        self.poly = poly
        self.seed = seed
        self.length = length

    def decode(self, inp):  # pylint: disable=no-self-use
        """
        Scrambler method

        :param inp: the data to be processed

        :returns: the scrambled/descrambled data
        """

        out = bytearray(
            additive_scramble(bytearray(inp), self.poly, self.seed,
                              self.length))
        return out



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Qubik(KaitaiStruct):
    """:field channel_id: osdlp_tm_frame.tm_tf_primary_header.channel_id
    :field master_channel_frame_count: osdlp_tm_frame.tm_tf_primary_header.master_channel_frame_count
    :field virtual_channel_frame_count: osdlp_tm_frame.tm_tf_primary_header.virtual_channel_frame_count
    :field transfer_frame_data_field_status: osdlp_tm_frame.tm_tf_primary_header.transfer_frame_data_field_status
    :field master_channel_id: osdlp_tm_frame.tm_tf_primary_header.master_channel_id
    :field tf_version_number: osdlp_tm_frame.tm_tf_primary_header.tf_version_number
    :field spacecraft_id: osdlp_tm_frame.tm_tf_primary_header.spacecraft_id
    :field virtual_channel_id: osdlp_tm_frame.tm_tf_primary_header.virtual_channel_id
    :field ocf_flag: osdlp_tm_frame.tm_tf_primary_header.ocf_flag
    :field tf_secondary_header_flag: osdlp_tm_frame.tm_tf_primary_header.tf_secondary_header_flag
    :field synch_flag: osdlp_tm_frame.tm_tf_primary_header.synch_flag
    :field packet_order_flag: osdlp_tm_frame.tm_tf_primary_header.packet_order_flag
    :field segment_length_id: osdlp_tm_frame.tm_tf_primary_header.segment_length_id
    :field first_header_pointer: osdlp_tm_frame.tm_tf_primary_header.first_header_pointer
    :field length: osdlp_tm_frame.tm_tf_data_field.payload.length
    :field curr_ins: osdlp_tm_frame.tm_tf_data_field.payload.curr_ins
    :field volt_ins: osdlp_tm_frame.tm_tf_data_field.payload.volt_ins
    :field temp_die: osdlp_tm_frame.tm_tf_data_field.payload.temp_die
    :field temp_bat: osdlp_tm_frame.tm_tf_data_field.payload.temp_bat
    :field curr_min: osdlp_tm_frame.tm_tf_data_field.payload.curr_min
    :field volt_min: osdlp_tm_frame.tm_tf_data_field.payload.volt_min
    :field temp_bat_min: osdlp_tm_frame.tm_tf_data_field.payload.temp_bat_min
    :field curr_max: osdlp_tm_frame.tm_tf_data_field.payload.curr_max
    :field volt_max: osdlp_tm_frame.tm_tf_data_field.payload.volt_max
    :field temp_bat_max: osdlp_tm_frame.tm_tf_data_field.payload.temp_bat_max
    :field curr_av: osdlp_tm_frame.tm_tf_data_field.payload.curr_av
    :field volt_av: osdlp_tm_frame.tm_tf_data_field.payload.volt_av
    :field temp_bat_av: osdlp_tm_frame.tm_tf_data_field.payload.temp_bat_av
    :field time_empty: osdlp_tm_frame.tm_tf_data_field.payload.time_empty
    :field cap_remain: osdlp_tm_frame.tm_tf_data_field.payload.cap_remain
    :field cap_full: osdlp_tm_frame.tm_tf_data_field.payload.cap_full
    :field charge_state: osdlp_tm_frame.tm_tf_data_field.payload.charge_state
    :field ant_test_volt: osdlp_tm_frame.tm_tf_data_field.payload.ant_test_volt
    :field ant_test_curr: osdlp_tm_frame.tm_tf_data_field.payload.ant_test_curr
    :field ant_test_res: osdlp_tm_frame.tm_tf_data_field.payload.ant_test_res
    :field ant_avg_volt: osdlp_tm_frame.tm_tf_data_field.payload.ant_avg_volt
    :field ant_avg_curr: osdlp_tm_frame.tm_tf_data_field.payload.ant_avg_curr
    :field ant_dep_duration: osdlp_tm_frame.tm_tf_data_field.payload.ant_dep_duration
    :field ant_dep_status: osdlp_tm_frame.tm_tf_data_field.payload.ant_dep_status
    :field ant_dep_retries: osdlp_tm_frame.tm_tf_data_field.payload.ant_dep_retries
    :field uptime: osdlp_tm_frame.tm_tf_data_field.payload.uptime
    :field temp_internal: osdlp_tm_frame.tm_tf_data_field.payload.temp_internal
    :field rtc_bat_volt: osdlp_tm_frame.tm_tf_data_field.payload.rtc_bat_volt
    :field deploy_first: osdlp_tm_frame.tm_tf_data_field.payload.deploy_first
    :field deploy_first_ant: osdlp_tm_frame.tm_tf_data_field.payload.deploy_first_ant
    :field ror_unknown: osdlp_tm_frame.tm_tf_data_field.payload.ror_unknown
    :field ror_low_power: osdlp_tm_frame.tm_tf_data_field.payload.ror_low_power
    :field ror_window_watchdog: osdlp_tm_frame.tm_tf_data_field.payload.ror_window_watchdog
    :field ror_independent_watchdog_reset: osdlp_tm_frame.tm_tf_data_field.payload.ror_independent_watchdog_reset
    :field ror_software: osdlp_tm_frame.tm_tf_data_field.payload.ror_software
    :field ror_power_on_power_down: osdlp_tm_frame.tm_tf_data_field.payload.ror_power_on_power_down
    :field ror_external_reset_pin: osdlp_tm_frame.tm_tf_data_field.payload.ror_external_reset_pin
    :field ror_brownout: osdlp_tm_frame.tm_tf_data_field.payload.ror_brownout
    :field dropped_frames: osdlp_tm_frame.tm_tf_data_field.payload.dropped_frames
    :field rorc_low_power: osdlp_tm_frame.tm_tf_data_field.payload.rorc_low_power
    :field rorc_ind_wdg: osdlp_tm_frame.tm_tf_data_field.payload.rorc_ind_wdg
    :field rorc_software: osdlp_tm_frame.tm_tf_data_field.payload.rorc_software
    :field rorc_brownout: osdlp_tm_frame.tm_tf_data_field.payload.rorc_brownout
    :field power_save: osdlp_tm_frame.tm_tf_data_field.payload.power_save
    :field power_monitor_fail: osdlp_tm_frame.tm_tf_data_field.payload.power_monitor_fail
    :field clcw_0_cw_type: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.cw_type
    :field clcw_0_clcw_ver: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.clcw_ver
    :field clcw_0_status_field: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.status_field
    :field clcw_0_cop: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.cop
    :field clcw_0_vcid: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.vcid
    :field clcw_0_rsvd_a: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.rsvd_a
    :field clcw_0_flag_norf: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.flag_norf
    :field clcw_0_flag_nobitlock: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.flag_nobitlock
    :field clcw_0_flag_lockout: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.flag_lockout
    :field clcw_0_flag_wait: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.flag_wait
    :field clcw_0_flag_retransmit: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.flag_retransmit
    :field clcw_0_farm_b_counter: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.farm_b_counter
    :field clcw_0_rsvd_b: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.rsvd_b
    :field clcw_0_report_value: osdlp_tm_frame.tm_tf_data_field.payload.clcw_0.report_value
    :field clcw_1_cw_type: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.cw_type
    :field clcw_1_clcw_ver: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.clcw_ver
    :field clcw_1_status_field: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.status_field
    :field clcw_1_cop: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.cop
    :field clcw_1_vcid: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.vcid
    :field clcw_1_rsvd_a: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.rsvd_a
    :field clcw_1_flag_norf: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.flag_norf
    :field clcw_1_flag_nobitlock: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.flag_nobitlock
    :field clcw_1_flag_lockout: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.flag_lockout
    :field clcw_1_flag_wait: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.flag_wait
    :field clcw_1_flag_retransmit: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.flag_retransmit
    :field clcw_1_farm_b_counter: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.farm_b_counter
    :field clcw_1_rsvd_b: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.rsvd_b
    :field clcw_1_report_value: osdlp_tm_frame.tm_tf_data_field.payload.clcw_1.report_value
    :field clcw_2_cw_type: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.cw_type
    :field clcw_2_clcw_ver: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.clcw_ver
    :field clcw_2_status_field: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.status_field
    :field clcw_2_cop: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.cop
    :field clcw_2_vcid: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.vcid
    :field clcw_2_rsvd_a: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.rsvd_a
    :field clcw_2_flag_norf: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.flag_norf
    :field clcw_2_flag_nobitlock: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.flag_nobitlock
    :field clcw_2_flag_lockout: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.flag_lockout
    :field clcw_2_flag_wait: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.flag_wait
    :field clcw_2_flag_retransmit: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.flag_retransmit
    :field clcw_2_farm_b_counter: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.farm_b_counter
    :field clcw_2_rsvd_b: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.rsvd_b
    :field clcw_2_report_value: osdlp_tm_frame.tm_tf_data_field.payload.clcw_2.report_value
    :field clcw_3_cw_type: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.cw_type
    :field clcw_3_clcw_ver: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.clcw_ver
    :field clcw_3_status_field: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.status_field
    :field clcw_3_cop: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.cop
    :field clcw_3_vcid: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.vcid
    :field clcw_3_rsvd_a: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.rsvd_a
    :field clcw_3_flag_norf: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.flag_norf
    :field clcw_3_flag_nobitlock: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.flag_nobitlock
    :field clcw_3_flag_lockout: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.flag_lockout
    :field clcw_3_flag_wait: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.flag_wait
    :field clcw_3_flag_retransmit: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.flag_retransmit
    :field clcw_3_farm_b_counter: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.farm_b_counter
    :field clcw_3_rsvd_b: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.rsvd_b
    :field clcw_3_report_value: osdlp_tm_frame.tm_tf_data_field.payload.clcw_3.report_value
    :field clcw_4_cw_type: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.cw_type
    :field clcw_4_clcw_ver: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.clcw_ver
    :field clcw_4_status_field: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.status_field
    :field clcw_4_cop: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.cop
    :field clcw_4_vcid: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.vcid
    :field clcw_4_rsvd_a: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.rsvd_a
    :field clcw_4_flag_norf: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.flag_norf
    :field clcw_4_flag_nobitlock: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.flag_nobitlock
    :field clcw_4_flag_lockout: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.flag_lockout
    :field clcw_4_flag_wait: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.flag_wait
    :field clcw_4_flag_retransmit: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.flag_retransmit
    :field clcw_4_farm_b_counter: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.farm_b_counter
    :field clcw_4_rsvd_b: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.rsvd_b
    :field clcw_4_report_value: osdlp_tm_frame.tm_tf_data_field.payload.clcw_4.report_value
    :field tx_frames: osdlp_tm_frame.tm_tf_data_field.payload.tx_frames
    :field rx_frames: osdlp_tm_frame.tm_tf_data_field.payload.rx_frames
    :field rx_corrected_bits: osdlp_tm_frame.tm_tf_data_field.payload.rx_corrected_bits
    :field rssi: osdlp_tm_frame.tm_tf_data_field.payload.rssi
    :field freq_tracking: osdlp_tm_frame.tm_tf_data_field.payload.freq_tracking
    :field agc: osdlp_tm_frame.tm_tf_data_field.payload.agc
    :field datarate_tracking: osdlp_tm_frame.tm_tf_data_field.payload.datarate_tracking
    :field phase_tracking: osdlp_tm_frame.tm_tf_data_field.payload.phase_tracking
    :field rf_freq_tracking: osdlp_tm_frame.tm_tf_data_field.payload.rf_freq_tracking
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.osdlp_tm_frame = Qubik.TmTransferFrameT(self._io, self, self._root)

    class VcidTmT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.length = self._io.read_u2be()
            self.curr_ins = self._io.read_s2be()
            self.volt_ins = self._io.read_u2be()
            self.temp_die = self._io.read_s1()
            self.temp_bat = self._io.read_s1()
            self.curr_min = self._io.read_s2be()
            self.volt_min = self._io.read_u2be()
            self.temp_bat_min = self._io.read_s1()
            self.curr_max = self._io.read_s2be()
            self.volt_max = self._io.read_u2be()
            self.temp_bat_max = self._io.read_s1()
            self.curr_av = self._io.read_s2be()
            self.volt_av = self._io.read_u2be()
            self.temp_bat_av = self._io.read_s1()
            self.time_empty = self._io.read_u2be()
            self.cap_remain = self._io.read_s2be()
            self.cap_full = self._io.read_s2be()
            self.charge_state = self._io.read_s1()
            self.ant_test_volt = self._io.read_s2be()
            self.ant_test_curr = self._io.read_s2be()
            self.ant_test_res = self._io.read_u1()
            self.ant_avg_volt = self._io.read_s2be()
            self.ant_avg_curr = self._io.read_s2be()
            self.ant_dep_duration = self._io.read_u1()
            self.ant_dep_status = self._io.read_u1()
            self.ant_dep_retries = self._io.read_u1()
            self.uptime = self._io.read_u4be()
            self.temp_internal = self._io.read_s1()
            self.rtc_bat_volt = self._io.read_u2be()
            self.deploy_first = self._io.read_u1()
            self.deploy_first_ant = self._io.read_u1()
            self.ror_unknown = self._io.read_bits_int_be(1) != 0
            self.ror_low_power = self._io.read_bits_int_be(1) != 0
            self.ror_window_watchdog = self._io.read_bits_int_be(1) != 0
            self.ror_independent_watchdog_reset = self._io.read_bits_int_be(1) != 0
            self.ror_software = self._io.read_bits_int_be(1) != 0
            self.ror_power_on_power_down = self._io.read_bits_int_be(1) != 0
            self.ror_external_reset_pin = self._io.read_bits_int_be(1) != 0
            self.ror_brownout = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.dropped_frames = self._io.read_u2be()
            self.rorc_low_power = self._io.read_u2be()
            self.rorc_ind_wdg = self._io.read_u2be()
            self.rorc_software = self._io.read_u2be()
            self.rorc_brownout = self._io.read_u2be()
            self.power_save = self._io.read_u1()
            self.power_monitor_fail = self._io.read_u1()
            self.clcw_0 = Qubik.Clcw(self._io, self, self._root)
            self.clcw_1 = Qubik.Clcw(self._io, self, self._root)
            self.clcw_2 = Qubik.Clcw(self._io, self, self._root)
            self.clcw_3 = Qubik.Clcw(self._io, self, self._root)
            self.clcw_4 = Qubik.Clcw(self._io, self, self._root)
            self.tx_frames = self._io.read_u2be()
            self.rx_frames = self._io.read_u2be()
            self.rx_corrected_bits = self._io.read_u4be()
            self.rssi = self._io.read_s1()
            self.freq_tracking = self._io.read_s2be()
            self.agc = self._io.read_s1()
            self.datarate_tracking = self._io.read_s4be()
            self.phase_tracking = self._io.read_s2be()
            self.rf_freq_tracking = self._io.read_s4be()


    class VcidExperimentT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes_full()


    class PrimaryHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.channel_id = self._io.read_u2be()
            self.master_channel_frame_count = self._io.read_u1()
            self.virtual_channel_frame_count = self._io.read_u1()
            self.transfer_frame_data_field_status = self._io.read_u2be()

        @property
        def synch_flag(self):
            if hasattr(self, '_m_synch_flag'):
                return self._m_synch_flag if hasattr(self, '_m_synch_flag') else None

            self._m_synch_flag = ((self.transfer_frame_data_field_status & 16384) >> 14)
            return self._m_synch_flag if hasattr(self, '_m_synch_flag') else None

        @property
        def spacecraft_id(self):
            if hasattr(self, '_m_spacecraft_id'):
                return self._m_spacecraft_id if hasattr(self, '_m_spacecraft_id') else None

            self._m_spacecraft_id = (self.master_channel_id & 255)
            return self._m_spacecraft_id if hasattr(self, '_m_spacecraft_id') else None

        @property
        def first_header_pointer(self):
            if hasattr(self, '_m_first_header_pointer'):
                return self._m_first_header_pointer if hasattr(self, '_m_first_header_pointer') else None

            self._m_first_header_pointer = (self.transfer_frame_data_field_status & 2047)
            return self._m_first_header_pointer if hasattr(self, '_m_first_header_pointer') else None

        @property
        def master_channel_id(self):
            if hasattr(self, '_m_master_channel_id'):
                return self._m_master_channel_id if hasattr(self, '_m_master_channel_id') else None

            self._m_master_channel_id = ((self.channel_id & 65520) >> 4)
            return self._m_master_channel_id if hasattr(self, '_m_master_channel_id') else None

        @property
        def packet_order_flag(self):
            if hasattr(self, '_m_packet_order_flag'):
                return self._m_packet_order_flag if hasattr(self, '_m_packet_order_flag') else None

            self._m_packet_order_flag = ((self.transfer_frame_data_field_status & 8192) >> 13)
            return self._m_packet_order_flag if hasattr(self, '_m_packet_order_flag') else None

        @property
        def tf_secondary_header_flag(self):
            if hasattr(self, '_m_tf_secondary_header_flag'):
                return self._m_tf_secondary_header_flag if hasattr(self, '_m_tf_secondary_header_flag') else None

            self._m_tf_secondary_header_flag = ((self.transfer_frame_data_field_status & 32768) >> 15)
            return self._m_tf_secondary_header_flag if hasattr(self, '_m_tf_secondary_header_flag') else None

        @property
        def tf_version_number(self):
            if hasattr(self, '_m_tf_version_number'):
                return self._m_tf_version_number if hasattr(self, '_m_tf_version_number') else None

            self._m_tf_version_number = ((self.master_channel_id & 3072) >> 2)
            return self._m_tf_version_number if hasattr(self, '_m_tf_version_number') else None

        @property
        def ocf_flag(self):
            if hasattr(self, '_m_ocf_flag'):
                return self._m_ocf_flag if hasattr(self, '_m_ocf_flag') else None

            self._m_ocf_flag = (self.channel_id & 1)
            return self._m_ocf_flag if hasattr(self, '_m_ocf_flag') else None

        @property
        def virtual_channel_id(self):
            if hasattr(self, '_m_virtual_channel_id'):
                return self._m_virtual_channel_id if hasattr(self, '_m_virtual_channel_id') else None

            self._m_virtual_channel_id = ((self.channel_id & 14) >> 1)
            return self._m_virtual_channel_id if hasattr(self, '_m_virtual_channel_id') else None

        @property
        def segment_length_id(self):
            if hasattr(self, '_m_segment_length_id'):
                return self._m_segment_length_id if hasattr(self, '_m_segment_length_id') else None

            self._m_segment_length_id = ((self.transfer_frame_data_field_status & 6144) >> 12)
            return self._m_segment_length_id if hasattr(self, '_m_segment_length_id') else None


    class Clcw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cw_type = self._io.read_bits_int_be(1) != 0
            self.clcw_ver = self._io.read_bits_int_be(2)
            self.status_field = self._io.read_bits_int_be(3)
            self.cop = self._io.read_bits_int_be(2)
            self.vcid = self._io.read_bits_int_be(6)
            self.rsvd_a = self._io.read_bits_int_be(2)
            self.flag_norf = self._io.read_bits_int_be(1) != 0
            self.flag_nobitlock = self._io.read_bits_int_be(1) != 0
            self.flag_lockout = self._io.read_bits_int_be(1) != 0
            self.flag_wait = self._io.read_bits_int_be(1) != 0
            self.flag_retransmit = self._io.read_bits_int_be(1) != 0
            self.farm_b_counter = self._io.read_bits_int_be(2)
            self.rsvd_b = self._io.read_bits_int_be(1) != 0
            self.report_value = self._io.read_bits_int_be(8)


    class TmTfDataFieldT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._parent.tm_tf_primary_header.virtual_channel_id
            if _on == 0:
                self._raw_payload = self._io.read_bytes(116)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Qubik.VcidManagementT(_io__raw_payload, self, self._root)
            elif _on == 4:
                self._raw_payload = self._io.read_bytes(116)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Qubik.VcidContactT(_io__raw_payload, self, self._root)
            elif _on == 1:
                self._raw_payload = self._io.read_bytes(116)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Qubik.VcidTmT(_io__raw_payload, self, self._root)
            elif _on == 3:
                self._raw_payload = self._io.read_bytes(116)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Qubik.VcidExperimentT(_io__raw_payload, self, self._root)
            elif _on == 2:
                self._raw_payload = self._io.read_bytes(116)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Qubik.VcidExtTmT(_io__raw_payload, self, self._root)
            else:
                self.payload = self._io.read_bytes(116)


    class VcidContactT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes_full()


    class VcidManagementT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes_full()


    class TmTransferFrameT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw_tm_tf_primary_header = self._io.read_bytes(6)
            _io__raw_tm_tf_primary_header = KaitaiStream(BytesIO(self._raw_tm_tf_primary_header))
            self.tm_tf_primary_header = Qubik.PrimaryHeaderT(_io__raw_tm_tf_primary_header, self, self._root)
            _on = self.tm_tf_primary_header.tf_secondary_header_flag
            if _on == 1:
                self.tf_secondary_header = self._io.read_u2be()
            self.tm_tf_data_field = Qubik.TmTfDataFieldT(self._io, self, self._root)
            self.tm_tf_trailer = self._io.read_bytes(6)
            self.unparsed = self._io.read_bytes_full()


    class VcidExtTmT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes_full()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Mysat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field callsign: ax25_frame.payload.ax25_info.beacon_type.callsign
    :field obc_mode: ax25_frame.payload.ax25_info.beacon_type.obc_mode
    :field obc_reset_counter: ax25_frame.payload.ax25_info.beacon_type.obc_reset_counter
    :field obc_uptime: ax25_frame.payload.ax25_info.beacon_type.obc_uptime
    :field gyro_norm: ax25_frame.payload.ax25_info.beacon_type.gyro_norm
    :field eps_reset_counter: ax25_frame.payload.ax25_info.beacon_type.eps_reset_counter
    :field eps_last_boot_cause: ax25_frame.payload.ax25_info.beacon_type.eps_last_boot_cause
    :field eps_battery_mode: ax25_frame.payload.ax25_info.beacon_type.eps_battery_mode
    :field timestamp: ax25_frame.payload.ax25_info.beacon_type.timestamp
    :field obc_temperature: ax25_frame.payload.ax25_info.beacon_type.obc_temperature
    :field obc_daughterboard_temperature: ax25_frame.payload.ax25_info.beacon_type.obc_daughterboard_temperature
    :field eps_battery_temperature: ax25_frame.payload.ax25_info.beacon_type.eps_battery_temperature
    :field eps_board_temperature: ax25_frame.payload.ax25_info.beacon_type.eps_board_temperature
    :field ants_temperature: ax25_frame.payload.ax25_info.beacon_type.ants_temperature
    :field trxvu_temperature: ax25_frame.payload.ax25_info.beacon_type.trxvu_temperature
    :field adcs_temperature: ax25_frame.payload.ax25_info.beacon_type.adcs_temperature
    :field obc_3v3_voltage: ax25_frame.payload.ax25_info.beacon_type.obc_3v3_voltage
    :field camera_voltage: ax25_frame.payload.ax25_info.beacon_type.camera_voltage
    :field trxvu_voltage: ax25_frame.payload.ax25_info.beacon_type.trxvu_voltage
    :field eps_battery_voltage: ax25_frame.payload.ax25_info.beacon_type.eps_battery_voltage
    :field obc_5v_current: ax25_frame.payload.ax25_info.beacon_type.obc_5v_current
    :field eps_total_pv_current: ax25_frame.payload.ax25_info.beacon_type.eps_total_pv_current
    :field eps_total_system_current: ax25_frame.payload.ax25_info.beacon_type.eps_total_system_current
    :field message: ax25_frame.payload.ax25_info.beacon_type.message
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    
    .. seealso::
       
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Mysat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Mysat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Mysat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Mysat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Mysat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Mysat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Mysat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Mysat.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Mysat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Mysat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Mysat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Mysat.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Mysat.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Mysat.MysatPayload(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Mysat.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Mysat.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Mysat.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class MysatMessage(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.message = (self._io.read_bytes_full()).decode(u"utf-8")


    class MysatPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._io.size()
            if _on == 42:
                self.beacon_type = Mysat.MysatTlm(self._io, self, self._root)
            else:
                self.beacon_type = Mysat.MysatMessage(self._io, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Mysat.Callsign(_io__raw_callsign_ror, self, self._root)


    class MysatTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(5)).decode(u"utf-8")
            self.obc_mode = self._io.read_u1()
            self.obc_reset_counter = self._io.read_u4le()
            self.obc_uptime = self._io.read_u4le()
            self.gyro_norm = self._io.read_u1()
            self.eps_reset_counter = self._io.read_u4le()
            self.eps_last_boot_cause = self._io.read_u1()
            self.eps_battery_mode = self._io.read_u1()
            self.timestamp = self._io.read_u4le()
            self.obc_temperature = self._io.read_u1()
            self.obc_daughterboard_temperature = self._io.read_u1()
            self.eps_battery_temperature = self._io.read_u1()
            self.eps_board_temperature = self._io.read_u1()
            self.ants_temperature = self._io.read_u1()
            self.trxvu_temperature = self._io.read_u1()
            self.adcs_temperature = self._io.read_u1()
            self.obc_3v3_voltage = self._io.read_u1()
            self.camera_voltage = self._io.read_u1()
            self.trxvu_voltage = self._io.read_u1()
            self.eps_battery_voltage = self._io.read_u1()
            self.obc_5v_current = self._io.read_u2le()
            self.eps_total_pv_current = self._io.read_u2le()
            self.eps_total_system_current = self._io.read_u2le()


    @property
    def frame_length(self):
        if hasattr(self, '_m_frame_length'):
            return self._m_frame_length if hasattr(self, '_m_frame_length') else None

        self._m_frame_length = self._io.size()
        return self._m_frame_length if hasattr(self, '_m_frame_length') else None





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Spoc(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field obc_curr_boot_image: ax25_frame.payload.ax25_info.obc_curr_boot_image
    :field mmm_mode: ax25_frame.payload.ax25_info.mmm_mode
    :field obt_uptime: ax25_frame.payload.ax25_info.obt_uptime
    :field bat_battery_current_dir: ax25_frame.payload.ax25_info.bat_battery_current_dir
    :field bat_battery_current_0: ax25_frame.payload.ax25_info.bat_battery_current_0
    :field bat_battery_current_1: ax25_frame.payload.ax25_info.bat_battery_current_1
    :field bat_battery_current_2: ax25_frame.payload.ax25_info.bat_battery_current_2
    :field bat_battery_voltage_0: ax25_frame.payload.ax25_info.bat_battery_voltage_0
    :field bat_battery_voltage_1: ax25_frame.payload.ax25_info.bat_battery_voltage_1
    :field bat_battery_voltage_2: ax25_frame.payload.ax25_info.bat_battery_voltage_2
    :field bat_battery_temperature_0: ax25_frame.payload.ax25_info.bat_battery_temperature_0
    :field bat_battery_temperature_1: ax25_frame.payload.ax25_info.bat_battery_temperature_1
    :field bat_battery_temperature_2: ax25_frame.payload.ax25_info.bat_battery_temperature_2
    :field bat_battery_temperature_3: ax25_frame.payload.ax25_info.bat_battery_temperature_3
    :field bat_battery_heater_status_0: ax25_frame.payload.ax25_info.bat_battery_heater_status_0
    :field bat_battery_heater_status_1: ax25_frame.payload.ax25_info.bat_battery_heater_status_1
    :field bat_battery_heater_status_2: ax25_frame.payload.ax25_info.bat_battery_heater_status_2
    :field bat_battery_heater_status_3: ax25_frame.payload.ax25_info.bat_battery_heater_status_3
    :field bat_status: ax25_frame.payload.ax25_info.bat_status
    :field bat_last_error: ax25_frame.payload.ax25_info.bat_last_error
    :field eps_status_0: ax25_frame.payload.ax25_info.eps_status_0
    :field eps_status_1: ax25_frame.payload.ax25_info.eps_status_1
    :field eps_last_error_0: ax25_frame.payload.ax25_info.eps_last_error_0
    :field eps_last_error_1: ax25_frame.payload.ax25_info.eps_last_error_1
    :field eps_switch_voltages_0: ax25_frame.payload.ax25_info.eps_switch_voltages_0
    :field eps_switch_voltages_1: ax25_frame.payload.ax25_info.eps_switch_voltages_1
    :field eps_switch_voltages_2: ax25_frame.payload.ax25_info.eps_switch_voltages_2
    :field eps_switch_voltages_3: ax25_frame.payload.ax25_info.eps_switch_voltages_3
    :field eps_switch_voltages_4: ax25_frame.payload.ax25_info.eps_switch_voltages_4
    :field eps_switch_voltages_5: ax25_frame.payload.ax25_info.eps_switch_voltages_5
    :field eps_switch_voltages_6: ax25_frame.payload.ax25_info.eps_switch_voltages_6
    :field eps_switch_voltages_7: ax25_frame.payload.ax25_info.eps_switch_voltages_7
    :field eps_switch_voltages_8: ax25_frame.payload.ax25_info.eps_switch_voltages_8
    :field eps_switch_voltages_9: ax25_frame.payload.ax25_info.eps_switch_voltages_9
    :field eps_switch_currents_0: ax25_frame.payload.ax25_info.eps_switch_currents_0
    :field eps_switch_currents_1: ax25_frame.payload.ax25_info.eps_switch_currents_1
    :field eps_switch_currents_2: ax25_frame.payload.ax25_info.eps_switch_currents_2
    :field eps_switch_currents_3: ax25_frame.payload.ax25_info.eps_switch_currents_3
    :field eps_switch_currents_4: ax25_frame.payload.ax25_info.eps_switch_currents_4
    :field eps_switch_currents_5: ax25_frame.payload.ax25_info.eps_switch_currents_5
    :field eps_switch_currents_6: ax25_frame.payload.ax25_info.eps_switch_currents_6
    :field eps_switch_currents_7: ax25_frame.payload.ax25_info.eps_switch_currents_7
    :field eps_switch_currents_8: ax25_frame.payload.ax25_info.eps_switch_currents_8
    :field eps_switch_currents_9: ax25_frame.payload.ax25_info.eps_switch_currents_9
    :field eps_expected_switch_states_bitmap: ax25_frame.payload.ax25_info.eps_expected_switch_states_bitmap
    :field eps_board_temperatures_0: ax25_frame.payload.ax25_info.eps_board_temperatures_0
    :field eps_board_temperatures_1: ax25_frame.payload.ax25_info.eps_board_temperatures_1
    :field eps_bus_voltages_0: ax25_frame.payload.ax25_info.eps_bus_voltages_0
    :field eps_bus_voltages_1: ax25_frame.payload.ax25_info.eps_bus_voltages_1
    :field eps_bus_voltages_2: ax25_frame.payload.ax25_info.eps_bus_voltages_2
    :field eps_bus_voltages_3: ax25_frame.payload.ax25_info.eps_bus_voltages_3
    :field eps_bus_currents_0: ax25_frame.payload.ax25_info.eps_bus_currents_0
    :field eps_bus_currents_1: ax25_frame.payload.ax25_info.eps_bus_currents_1
    :field eps_bus_currents_2: ax25_frame.payload.ax25_info.eps_bus_currents_2
    :field eps_bus_currents_3: ax25_frame.payload.ax25_info.eps_bus_currents_3
    :field cmc_rx_lock: ax25_frame.payload.ax25_info.cmc_rx_lock
    :field cmc_rx_frame_count: ax25_frame.payload.ax25_info.cmc_rx_frame_count
    :field cmc_rx_packet_count: ax25_frame.payload.ax25_info.cmc_rx_packet_count
    :field cmc_rx_dropped_error_count: ax25_frame.payload.ax25_info.cmc_rx_dropped_error_count
    :field cmc_rx_crc_error_count: ax25_frame.payload.ax25_info.cmc_rx_crc_error_count
    :field cmc_rx_overrun_error_count: ax25_frame.payload.ax25_info.cmc_rx_overrun_error_count
    :field cmc_rx_protocol_error_count: ax25_frame.payload.ax25_info.cmc_rx_protocol_error_count
    :field cmc_temperature_smps: ax25_frame.payload.ax25_info.cmc_temperature_smps
    :field cmc_temperature_pa: ax25_frame.payload.ax25_info.cmc_temperature_pa
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Spoc.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Spoc.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Spoc.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Spoc.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Spoc.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Spoc.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Spoc.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Spoc.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Spoc.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Spoc.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Spoc.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Spoc.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Spoc.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self._parent.ax25_header.src_callsign_raw.callsign_ror.callsign
            if _on == u"MOCI  ":
                self.ax25_info = Spoc.SpocTlm(self._io, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class SpocTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.offset = self._io.read_bytes(23)
            self.obc_curr_boot_image = self._io.read_bits_int_be(8)
            self.mmm_mode = self._io.read_bits_int_be(3)
            self.obt_uptime = self._io.read_bits_int_be(32)
            self.bat_battery_current_dir = self._io.read_bits_int_be(1) != 0
            self.bat_battery_current_0 = self._io.read_bits_int_be(10)
            self.bat_battery_current_1 = self._io.read_bits_int_be(10)
            self.bat_battery_current_2 = self._io.read_bits_int_be(10)
            self.bat_battery_voltage_0 = self._io.read_bits_int_be(10)
            self.bat_battery_voltage_1 = self._io.read_bits_int_be(10)
            self.bat_battery_voltage_2 = self._io.read_bits_int_be(10)
            self.bat_battery_temperature_0 = self._io.read_bits_int_be(10)
            self.bat_battery_temperature_1 = self._io.read_bits_int_be(10)
            self.bat_battery_temperature_2 = self._io.read_bits_int_be(10)
            self.bat_battery_temperature_3 = self._io.read_bits_int_be(10)
            self.bat_battery_heater_status_0 = self._io.read_bits_int_be(10)
            self.bat_battery_heater_status_1 = self._io.read_bits_int_be(10)
            self.bat_battery_heater_status_2 = self._io.read_bits_int_be(10)
            self.bat_battery_heater_status_3 = self._io.read_bits_int_be(10)
            self.bat_status = self._io.read_bits_int_be(16)
            self.bat_last_error = self._io.read_bits_int_be(8)
            self.eps_status_0 = self._io.read_bits_int_be(16)
            self.eps_status_1 = self._io.read_bits_int_be(16)
            self.eps_last_error_0 = self._io.read_bits_int_be(16)
            self.eps_last_error_1 = self._io.read_bits_int_be(16)
            self.eps_switch_voltages_0 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_1 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_2 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_3 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_4 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_5 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_6 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_7 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_8 = self._io.read_bits_int_be(10)
            self.eps_switch_voltages_9 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_0 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_1 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_2 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_3 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_4 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_5 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_6 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_7 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_8 = self._io.read_bits_int_be(10)
            self.eps_switch_currents_9 = self._io.read_bits_int_be(10)
            self.eps_expected_switch_states_bitmap = self._io.read_bits_int_be(10)
            self.eps_board_temperatures_0 = self._io.read_bits_int_be(10)
            self.eps_board_temperatures_1 = self._io.read_bits_int_be(10)
            self.eps_bus_voltages_0 = self._io.read_bits_int_be(10)
            self.eps_bus_voltages_1 = self._io.read_bits_int_be(10)
            self.eps_bus_voltages_2 = self._io.read_bits_int_be(10)
            self.eps_bus_voltages_3 = self._io.read_bits_int_be(10)
            self.eps_bus_currents_0 = self._io.read_bits_int_be(10)
            self.eps_bus_currents_1 = self._io.read_bits_int_be(10)
            self.eps_bus_currents_2 = self._io.read_bits_int_be(10)
            self.eps_bus_currents_3 = self._io.read_bits_int_be(10)
            self.cmc_rx_lock = self._io.read_bits_int_be(1) != 0
            self.cmc_rx_frame_count = self._io.read_bits_int_be(16)
            self.cmc_rx_packet_count = self._io.read_bits_int_be(16)
            self.cmc_rx_dropped_error_count = self._io.read_bits_int_be(16)
            self.cmc_rx_crc_error_count = self._io.read_bits_int_be(16)
            self.cmc_rx_overrun_error_count = self._io.read_bits_int_be(8)
            self.cmc_rx_protocol_error_count = self._io.read_bits_int_be(16)
            self._io.align_to_byte()
            self.cmc_temperature_smps = self._io.read_s1()
            self.cmc_temperature_pa = self._io.read_s1()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Spoc.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Spoc.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Spoc.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Spoc.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Minxss(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field spacecraft_flags: ax25_frame.payload.ax25_info.beacon.spacecraft_flags
    :field pointing_mode: ax25_frame.payload.ax25_info.beacon.pointing_mode
    :field cmd_accept_cnt: ax25_frame.payload.ax25_info.beacon.cmd_accept_cnt
    :field flight_model: ax25_frame.payload.ax25_info.beacon.flight_model
    :field cdh_board_temp: ax25_frame.payload.ax25_info.beacon.cdh_board_temp
    :field enable_flags: ax25_frame.payload.ax25_info.beacon.enable_flags
    :field comm_brd_temp: ax25_frame.payload.ax25_info.beacon.comm_brd_temp
    :field mbrd_temp: ax25_frame.payload.ax25_info.beacon.mbrd_temp
    :field eps_brd_temp: ax25_frame.payload.ax25_info.beacon.eps_brd_temp
    :field battery_voltage: ax25_frame.payload.ax25_info.beacon.battery_voltage
    :field solar_panel_minus_y_curr: ax25_frame.payload.ax25_info.beacon.solar_panel_minus_y_curr
    :field solar_panel_minus_y_volt: ax25_frame.payload.ax25_info.beacon.solar_panel_minus_y_volt
    :field solar_panel_plus_x_curr: ax25_frame.payload.ax25_info.beacon.solar_panel_plus_x_curr
    :field solar_panel_plus_x_volt: ax25_frame.payload.ax25_info.beacon.solar_panel_plus_x_volt
    :field solar_panel_plus_y_curr: ax25_frame.payload.ax25_info.beacon.solar_panel_plus_y_curr
    :field solar_panel_plus_y_volt: ax25_frame.payload.ax25_info.beacon.solar_panel_plus_y_volt
    :field solar_panel_minus_y_temp: ax25_frame.payload.ax25_info.beacon.solar_panel_minus_y_temp
    :field solar_panel_plus_x_temp: ax25_frame.payload.ax25_info.beacon.solar_panel_plus_x_temp
    :field solar_panel_plus_y_temp: ax25_frame.payload.ax25_info.beacon.solar_panel_plus_y_temp
    :field battery_chrg_curr: ax25_frame.payload.ax25_info.beacon.battery_chrg_curr
    :field battery_dchrg_curr: ax25_frame.payload.ax25_info.beacon.battery_dchrg_curr
    :field battery_temp: ax25_frame.payload.ax25_info.beacon.battery_temp
    :field xp: ax25_frame.payload.ax25_info.beacon.xp
    :field sps_x: ax25_frame.payload.ax25_info.beacon.sps_x
    :field sps_y: ax25_frame.payload.ax25_info.beacon.sps_y
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Minxss.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Minxss.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Minxss.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Minxss.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Minxss.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Minxss.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Minxss.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Minxss.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Minxss.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Minxss.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Minxss.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Minxss.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Minxss.Telemetry(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class HskpData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.res_1 = [None] * (10)
            for i in range(10):
                self.res_1[i] = self._io.read_u1()

            self.spacecraft_flags = self._io.read_u1()
            self.pointing_mode = self._io.read_u1()
            self.res_2 = self._io.read_u2be()
            self.cmd_accept_cnt = self._io.read_u2le()
            self.res_3 = [None] * (33)
            for i in range(33):
                self.res_3[i] = self._io.read_u1()

            self.flight_model = self._io.read_u1()
            self.res_4 = [None] * (34)
            for i in range(34):
                self.res_4[i] = self._io.read_u1()

            self.cdh_board_temp = self._io.read_s2le()
            self.enable_flags = self._io.read_u2be()
            self.res_5 = [None] * (32)
            for i in range(32):
                self.res_5[i] = self._io.read_u1()

            self.comm_brd_temp = self._io.read_s2le()
            self.mbrd_temp = self._io.read_s2le()
            self.res_6 = self._io.read_u2be()
            self.eps_brd_temp = self._io.read_s2le()
            self.res_7 = self._io.read_u2be()
            self.battery_voltage = self._io.read_u2le()
            self.res_8 = self._io.read_u2be()
            self.solar_panel_minus_y_curr = self._io.read_u2le()
            self.solar_panel_minus_y_volt = self._io.read_u2le()
            self.solar_panel_plus_x_curr = self._io.read_u2le()
            self.solar_panel_plus_x_volt = self._io.read_u2le()
            self.solar_panel_plus_y_curr = self._io.read_u2le()
            self.solar_panel_plus_y_volt = self._io.read_u2le()
            self.res_9 = [None] * (12)
            for i in range(12):
                self.res_9[i] = self._io.read_u1()

            self.solar_panel_minus_y_temp = self._io.read_u2le()
            self.solar_panel_plus_x_temp = self._io.read_u2le()
            self.solar_panel_plus_y_temp = self._io.read_u2le()
            self.res_0 = self._io.read_u2be()
            self.battery_chrg_curr = self._io.read_u2le()
            self.res_a = self._io.read_u2be()
            self.battery_dchrg_curr = self._io.read_u2le()
            self.battery_temp = self._io.read_u2le()
            self.res_b = [None] * (16)
            for i in range(16):
                self.res_b[i] = self._io.read_u1()

            self.xp = self._io.read_u4le()
            self.res_c = [None] * (10)
            for i in range(10):
                self.res_c[i] = self._io.read_u1()

            self.sps_x = self._io.read_u2le()
            self.sps_y = self._io.read_u2le()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class RtData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rt_data_raw = self._io.read_bytes((self._io.size() - 4))


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Minxss.Callsign(_io__raw_callsign_ror, self, self._root)


    class Telemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.start_sync_flag = self._io.read_u2be()
            _on = self.start_sync_flag
            if _on == 2073:
                self._raw_beacon = self._io.read_bytes((self._io.size() - 4))
                _io__raw_beacon = KaitaiStream(BytesIO(self._raw_beacon))
                self.beacon = Minxss.HskpData(_io__raw_beacon, self, self._root)
            elif _on == 2077:
                self._raw_beacon = self._io.read_bytes((self._io.size() - 4))
                _io__raw_beacon = KaitaiStream(BytesIO(self._raw_beacon))
                self.beacon = Minxss.RtData(_io__raw_beacon, self, self._root)
            else:
                self.beacon = self._io.read_bytes((self._io.size() - 4))
            self.stop_sync_flag = self._io.read_bytes(2)
            if not self.stop_sync_flag == b"\xA5\xA5":
                raise kaitaistruct.ValidationNotEqualError(b"\xA5\xA5", self.stop_sync_flag, self._io, u"/types/telemetry/seq/2")



